'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _system = require('system');

var _system2 = _interopRequireDefault(_system);

var _webpage = require('webpage');

var _webpage2 = _interopRequireDefault(_webpage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// n.b. Phantom's require uses a different resolver, so we concat to libraryPath
var Logger = require(phantom.libraryPath + '/utils/logger').default;
var PhantomError = require(phantom.libraryPath + '/utils/error').default;

var logger = new Logger('Page');

var Page = function () {
  function Page() {
    var _this = this;

    _classCallCheck(this, Page);

    this.page = _webpage2.default.create();
    this.canPrint = false;

    this.page.onError = function (msg, trace) {
      var error = new PhantomError(msg, trace);
      logger.error(error);
    };

    this.page.onLoadFinished = function () {
      _this.canPrint = true;
    };
  }

  _createClass(Page, [{
    key: 'load',
    value: function load(_ref) {
      var url = _ref.url,
          html = _ref.html;

      this.canPrint = false;

      if (html && html.constructor === String) {
        this.page.setContent(html, null);
      } else if (url && url.constructor === String) {
        this.page.open(url);
      } else {
        throw new Error('must pass either `url` or `html` option');
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          filename = _ref2.filename,
          _ref2$type = _ref2.type,
          type = _ref2$type === undefined ? 'pdf' : _ref2$type,
          _ref2$quality = _ref2.quality,
          quality = _ref2$quality === undefined ? '75' : _ref2$quality,
          _ref2$paperOptions = _ref2.paperOptions,
          paperOptions = _ref2$paperOptions === undefined ? {} : _ref2$paperOptions;

      filename = filename || '/tmp/render-vendor-' + _system2.default.pid + '/' + Date.now() + '.' + type;

      if (!this.canPrint) {
        throw new Error('no can print right now');
      }

      // n.b. paperSize must be set before render
      this.page.paperSize = Page.computePaperSize(this.page, paperOptions);
      this.page.render(filename, { type: type, quality: quality });

      return filename;
    }
  }]);

  return Page;
}();

exports.default = Page;


Page.computePaperSize = function computePaperSize(page, options) {
  var content = Page.getPageContent(page);
  var paper = { margin: options.margin || '0' };

  if (options.height !== undefined && options.width !== undefined) {
    paper.width = options.width;
    paper.height = options.height;
  } else if (options.isSinglePage === true) {
    var _page$evaluate = page.evaluate(function (heightMultiplier) {
      var body = document.getElementsByTagName('body')[0];

      return {
        width: body.scrollWidth,
        height: body.scrollHeight * heightMultiplier
      };
    }, options.singlePageHeightMultiplier || 1.31),
        width = _page$evaluate.width,
        height = _page$evaluate.height;

    paper.width = options.width || width;
    paper.height = options.height || height;
  } else {
    paper.format = options.format || 'A4';
    paper.orientation = options.orientation || 'portrait';
  }

  Array.prototype.forEach.call(['header', 'footer'], function (name) {
    if (options[name] || content[name]) {
      var defaultContent = content[name] || {};
      var option = options[name] || {};
      var styles = content.styles;

      paper[name] = Page.getPageSection(defaultContent, option, styles);

      if (paper[name].height === undefined) {
        paper[name].height = name === 'header' ? '46mm' : '28mm';
      }
    }
  });

  return paper;
};

Page.getPageContent = function getPageContent(page) {
  return page.evaluate(function () {
    var getElements = function getElements(doc, wildcard) {
      var wildcardMatcher = new RegExp(wildcard + '(.*)');
      var hasElements = false;
      var elements = {};
      var $elements = doc.querySelectorAll('[id*=\'' + wildcard + '\']');

      Array.prototype.forEach.call($elements, function ($elem) {
        var match = $elem.attributes.id.value.match(wildcardMatcher);

        if (match) {
          hasElements = true;
          elements[match[1]] = $elem.outerHTML;
          $elem.parentNode.removeChild($elem);
        }
      });

      return hasElements ? elements : {};
    };

    var getElement = function getElement(doc, id) {
      var $elem = doc.getElementById(id);
      var html = void 0;

      if ($elem) {
        html = $elem.outerHTML;
        $elem.parentNode.removeChild($elem);
      }

      return html;
    };

    var styles = Array.prototype.reduce.call(document.querySelectorAll('link,style'), function (string, node) {
      return string + (node.outerHTML || '');
    }, '');

    // Wildcard headers e.g. <div id="pageHeader-[first|0]">
    var header = getElements(document, 'pageHeader-');
    var footer = getElements(document, 'pageFooter-');

    // Default header and footer e.g. <div id="pageHeader">
    var h = getElement(document, 'pageHeader');
    var f = getElement(document, 'pageFooter');

    var $body = document.getElementById('pageContent');
    var body = $body !== null ? $body.outerHTML : document.body.outerHTML;

    if (h) {
      header.default = h;
    }

    if (f) {
      footer.default = f;
    }

    return { styles: styles, header: header, body: body, footer: footer };
  });
};

Page.getPageSection = function getPageSection(defaultContent, option, styles) {
  var optionContent = option.contents;

  if ((typeof optionContent === 'undefined' ? 'undefined' : _typeof(optionContent)) !== 'object') {
    optionContent = { default: optionContent };
  }

  function readSectionValue(key) {
    return optionContent[key] || defaultContent[key];
  }

  return {
    height: option.height,
    contents: phantom.callback(function (pageNum, numPages) {
      var html = readSectionValue(pageNum);

      if (pageNum === 1 && !html) {
        html = readSectionValue('first');
      }

      if (pageNum === numPages && !html) {
        html = readSectionValue('last');
      }

      return (html || readSectionValue(pageNum) || '').replace(/\$page/g, pageNum).replace(/\$numPages/g, numPages).concat(styles);
    })
  };
};